package de.hdm_stuttgart.battlearena.Model.Entity;

import de.hdm_stuttgart.battlearena.Controller.Enum.PlayerMode;
import de.hdm_stuttgart.battlearena.Controller.GameSceneController;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.Bomb;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.Heart;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.ObjectType;
import de.hdm_stuttgart.battlearena.Model.Inputs.InputHandler;

import javafx.scene.canvas.GraphicsContext;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

public class EntityFactory {

    private static final Logger log = LogManager.getLogger(EntityFactory.class);

    public static IEntity createEntity(EntityType entityType, GraphicsContext gameScene, InputHandler inputHandler,
                                       EntityClass entityClass, GameSceneController gameSceneController, PlayerMode playerMode) {
        if (entityType == EntityType.PLAYER) {
            log.debug("Entity " + entityType + " created");
            return new Player(gameScene, inputHandler, entityClass, gameSceneController, playerMode);
        } else if (entityType == EntityType.NETWORK_PLAYER_TWO) {
            log.debug("Entity " + entityType + " created");
            return new NetworkPlayerTwo();
        }

        throw new IllegalArgumentException ("Entity type not supported " + entityType);
    }

    public static IEntity createGameplayObject(ObjectType objectType, int cordX, int cordY, GraphicsContext graphicsContext) {
        if (objectType == ObjectType.HEART) {
            return new Heart(cordX, cordY, graphicsContext);
        } else if (objectType == ObjectType.BIG_BOMB) {
            return new Bomb(cordX, cordY, graphicsContext);
        } else if (objectType == ObjectType.BOMB) {
            return new Bomb(cordX, cordY, graphicsContext);
        } else if (objectType == ObjectType.EXPLOSION) {
            return new Explosion(cordX, cordY, graphicsContext);
        }
        throw new IllegalArgumentException();
    }

}