package de.hdm_stuttgart.battlearena.Model.Entity;

import de.hdm_stuttgart.battlearena.Controller.Enum.PlayerMode;
import de.hdm_stuttgart.battlearena.Controller.GameSceneController;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.RuntimeInfo;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.ObjectStatus;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.ObjectType;
import de.hdm_stuttgart.battlearena.Model.Inputs.InputHandler;
import de.hdm_stuttgart.battlearena.Model.Map.TileManager;

import de.hdm_stuttgart.battlearena.Model.Sound.SFX;
import de.hdm_stuttgart.battlearena.Model.Sound.SoundEffects;
import javafx.geometry.BoundingBox;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import java.util.List;
import java.util.Objects;

class Player implements IEntity {

    private static final Logger log = LogManager.getLogger(Player.class);

    private final PlayerMode PLAYER_MODE;

    SoundEffects soundEffects = new SoundEffects();

    InputHandler inputHandler;

    RuntimeInfo runtimeInfo = RuntimeInfo.getInstance();

    CollisionHandler collisionHandler = new CollisionHandler();

    GraphicsContext gameScene;

    GameSceneController gameSceneController;

    private BoundingBox boxCollider;

    private Image directionDownOne,
            directionDownTwo,
            directionUpOne,
            directionUpTwo,
            directionLeftOne,
            directionLeftTwo,
            directionRightOne,
            directionRightTwo;

    private Image swordDown,
            swordUp,
            swordLeft,
            swordRight;

    private final EntityClass entityClass;

    private int spriteCounter = 0;
    private int spriteNumber = 1;

    private final int playerWidth = 19;
    private final int playerHeight = 20;

    private int scaledTileSize;

    private int mapPosX;
    private int mapPosY;

    private int lastMapPosX;
    private int lastMapPosY;

    private int playerSpeed;
    private EntityDirection playerDirection;
    private ObjectStatus objectStatus = ObjectStatus.UNUSED;

    private int health;
    private int maxPlayerHealth;
    private int damage;

    public Player(GraphicsContext gameScene, InputHandler inputHandler, EntityClass entityClass,
                  GameSceneController gameSceneController, PlayerMode PLAYER_MODE) {
        this.gameScene = gameScene;
        this.inputHandler = inputHandler;
        this.entityClass = entityClass;
        this.gameSceneController = gameSceneController;
        this.PLAYER_MODE = PLAYER_MODE;

        initializeEntity();
        loadEntitySprites();
    }

    @Override
    public void initializeEntity() {
        scaledTileSize = gameSceneController.getScaledTileSize();
        if (PLAYER_MODE == PlayerMode.PLAYER_ONE) {
            mapPosX = 60;
            mapPosY = 60;
        } else if (PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            mapPosX = 760;
            mapPosY = 735;
        }

        initializePlayerStats();

        boxCollider = new BoundingBox(mapPosX + 15, mapPosY + 10, playerWidth, playerHeight);
        playerDirection = EntityDirection.DOWN;
    }

    //Values result from balancing
    private void initializePlayerStats() {
        if (entityClass == EntityClass.HUMAN) {
            maxPlayerHealth = 50;
            health = 50;
            damage = 5;
            playerSpeed = 1; //5
        } else if (entityClass == EntityClass.HIGH_BORN) {
            maxPlayerHealth = 100;
            health = 75;
            damage = 2;
            playerSpeed = 1; //3
        } else if (entityClass == EntityClass.LOW_BORN) {
            maxPlayerHealth = 10;
            health = 10;
            damage = 10;
            playerSpeed = 1; //7
        } else {
            maxPlayerHealth = 40;
            health = 15;
            damage = 7;
            playerSpeed = 1; //5
        }
    }

    @Override
    public void loadEntitySprites() {
        try {
            if (entityClass == EntityClass.HUMAN) {
                loadHumanSprites();
                loadWeaponSprites();
            } else if (entityClass == EntityClass.HIGH_BORN) {
                loadHighBornSprites();
                loadWeaponSprites();
            } else if (entityClass == EntityClass.LOW_BORN) {
                loadLowBornSprites();
                loadWeaponSprites();
            } else {
                loadSentinelsSprites();
                loadWeaponSprites();
            }
        } catch (Exception e) {
            log.error(e);
        }
    }

    private void loadHumanSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanUp00.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanUp01.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanLeft00.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanLeft01.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanRight00.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanRight01.png")));
    }

    private void loadHighBornSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/high_born/HighBornDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/high_born/HighBornDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadLowBornSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/HumanDown01.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/HumanDown02.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadSentinelsSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/sentinels/SentinelDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/sentinels/SentinelDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadWeaponSprites() {
        swordUp = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordUp.png")));
        swordDown = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordDown.png")));
        swordLeft = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordLeft.png")));
        swordRight = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordRight.png")));
    }

    @Override
    public void updateEntityMovement(GameSceneController gameScene) {
        if (PLAYER_MODE == PlayerMode.PLAYER_ONE) {
            if (inputHandler.isMoveUp() || inputHandler.isMoveDown() || inputHandler.isMoveLeft() || inputHandler.isMoveRight()) {
                if (inputHandler.isMoveUp()) {
                    playerDirection = EntityDirection.UP;
                } else if (inputHandler.isMoveDown()) {
                    playerDirection = EntityDirection.DOWN;
                } else if (inputHandler.isMoveLeft()) {
                    playerDirection = EntityDirection.LEFT;
                } else if (inputHandler.isMoveRight()) {
                    playerDirection = EntityDirection.RIGHT;
                }

                boolean isWalkableTile = collisionHandler.handleMapCollision(this, scaledTileSize, gameSceneController);

                performEntityMovement(isWalkableTile);

                boolean isEntityCollision = boxCollider.intersects(gameScene.getEnemy().getBoxCollider());
                //boolean isEntityCollision = collisionHandler.handleBoxCollision(boxCollider, gameScene.getEnemy().getBoxCollider());

                performEntityCollision(isEntityCollision);

                updateEntityWalkAnimation();
            }
        } else if (PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            if (inputHandler.isSdMoveUp() || inputHandler.isSdMoveDown() || inputHandler.isSdMoveLeft() || inputHandler.isSdMoveRight()) {
                if (inputHandler.isSdMoveUp()) {
                    playerDirection = EntityDirection.UP;
                } else if (inputHandler.isSdMoveDown()) {
                    playerDirection = EntityDirection.DOWN;
                } else if (inputHandler.isSdMoveLeft()) {
                    playerDirection = EntityDirection.LEFT;
                } else if (inputHandler.isSdMoveRight()) {
                    playerDirection = EntityDirection.RIGHT;
                }

                boolean isWalkableTile = collisionHandler.handleMapCollision(this, scaledTileSize, gameSceneController);

                performEntityMovement(isWalkableTile);

                boolean isEntityCollision = boxCollider.intersects(gameScene.getPlayer().getBoxCollider());
                //boolean isEntityCollision = collisionHandler.handleBoxCollision(boxCollider, gameScene.getEnemy().getBoxCollider());

                performEntityCollision(isEntityCollision);

                updateEntityWalkAnimation();
            }
        }
    }

    private void performEntityMovement(boolean isWalkableTile) {
        lastMapPosX = mapPosX;
        lastMapPosY = mapPosY;

        if (isWalkableTile) {
            switch (playerDirection) {
                case UP:
                    mapPosY -= playerSpeed;
                    break;
                case DOWN:
                    mapPosY += playerSpeed;
                    break;
                case LEFT:
                    mapPosX -= playerSpeed;
                    break;
                case RIGHT:
                    mapPosX += playerSpeed;
            }
        }
    }

    private void performEntityCollision(boolean isEntityCollision) {
        if (isEntityCollision) {
            if (playerDirection == EntityDirection.UP) {
                mapPosY = lastMapPosY + 30;
            } else if (playerDirection == EntityDirection.DOWN) {
                mapPosY = lastMapPosY - 30;
            } else if (playerDirection == EntityDirection.LEFT) {
                mapPosX = lastMapPosX + 30;
            } else {
                mapPosX = lastMapPosX - 30;
            }
        }

        boxCollider = new BoundingBox(mapPosX + 15, mapPosY + 20, playerWidth, playerHeight);
    }

    @Override
    public void checkGameplayObjectInteraction(IEntity entity, GraphicsContext graphicsContext) {

        List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
        for (IEntity gameplayObject : gameplayObjects) {
            if (gameplayObject.getOBJECT_TYPE() == ObjectType.HEART) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    log.info("Collision with heart");
                    healPlayer(5);
                    gameplayObject.setObjectStatus(ObjectStatus.USED);
                }
            } else if (gameplayObject.getOBJECT_TYPE() == ObjectType.BOMB) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    log.info("Collision with bomb");
                    gotHit(10);
                }
            } else if (gameplayObject.getOBJECT_TYPE() == ObjectType.EXPLOSION) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    log.info("Collision with explosion");
                    gotHit(10);
                }
            }
        }
    }

    @Override
    public void placeBomb(IEntity entity, IEntity entity2, GraphicsContext graphicsContext) {

        if (inputHandler.isBomb() && PLAYER_MODE == PlayerMode.PLAYER_ONE ||
                inputHandler.isSdBomb() && PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            soundEffects.playSoundEffectOnce(SFX.BOMB_FUSE);
            int spawnCordX = mapPosX + 12;
            int spawnCordY = mapPosY + 12;

            List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
            IEntity bomb = EntityFactory.createGameplayObject(ObjectType.BOMB, spawnCordX, spawnCordY, graphicsContext);
            gameplayObjects.add(bomb);
            runtimeInfo.setGameplayObjects(gameplayObjects);
            log.info("bomb placed");
        }
    }

    public void checkChangeTiles(IEntity entity) {
        boolean isDestructible;
        int xTile = entity.getMapPosX() / gameSceneController.getScaledTileSize();
        int yTile = entity.getMapPosY() / gameSceneController.getScaledTileSize();
        isDestructible = checkTileNon_Destructible(yTile, xTile);
        if (isDestructible) {
            TileManager.tileMap[yTile][xTile] = 7;
            for (int i = 0; i < 3; i++) {
                isDestructible = checkTileNon_Destructible(yTile + 1 + i, xTile);         //If Tile solid , -> should stop comparing,
                if (isDestructible && (yTile < 18 && xTile < 18)) {//because for now, Tile behind gets destroyed aswell
                    soundEffects.playSoundEffectOnce(SFX.WOOD_Destruction);
                    TileManager.tileMap[yTile + 1 + i][xTile] = 8;
                       /*if(entity || entity2) {
                           entity2.gotHit(2);    //See if enemy or player hit
                       }*/
                }
                isDestructible = checkTileNon_Destructible(yTile - 1 - i, xTile);
                if (isDestructible && (yTile < 18 && xTile < 18 && yTile > 0 && xTile > 0)) {
                    soundEffects.playSoundEffectOnce(SFX.WOOD_Destruction);
                    TileManager.tileMap[yTile - 1 - i][xTile] = 8;
                    entity.gotHit(2);
                }
                isDestructible = checkTileNon_Destructible(yTile, xTile + 1 + i);
                if (isDestructible && (yTile < 18 && xTile < 18 && yTile > 0 && xTile > 0)) {
                    soundEffects.playSoundEffectOnce(SFX.WOOD_Destruction);
                    TileManager.tileMap[yTile][xTile + 1 + i] = 8;
                    entity.gotHit(2);
                }
                isDestructible = checkTileNon_Destructible(yTile, xTile - 1 - i);
                if (isDestructible && (yTile < 18 && xTile < 18 && yTile > 0 && xTile > 0)) {
                    soundEffects.playSoundEffectOnce(SFX.WOOD_Destruction);
                    TileManager.tileMap[yTile][xTile - 1 - i] = 8;
                    entity.gotHit(2);
                }
            }
        }
    }

            /*isDestructible = checkTileNon_Destructible(yTile, xTile);   //for loop
            if(isDestructible){
                TileManager.tileMap[yTile][xTile] = 7;
                 isDestructible = checkTileNon_Destructible(yTile-1, xTile);   //for loop
            }else if(isDestructible)
            TileManager.tileMap[yTile-1][xTile] = 8;
            TileManager.tileMap[yTile-2][xTile] = 8;        //Change it with a DestructionHandler like CollisionHandler
            TileManager.tileMap[yTile+1][xTile] = 8;
            TileManager.tileMap[yTile+2][xTile] = 8;        //Why doesnt it work like in CollisionHandler with tileSet
            TileManager.tileMap[yTile][xTile+1] = 8;
            TileManager.tileMap[yTile][xTile+2] = 8;
            TileManager.tileMap[yTile][xTile-1] = 8;
            TileManager.tileMap[yTile][xTile-2] = 8;*/
    //}
    //}

    public boolean checkTileNon_Destructible(int y, int x) {     //Change it with a DestructionHandler like CollisionHandler
        //tileSet[xTile].getDestruction();
        if (TileManager.tileMap[y][x] != 10 && TileManager.tileMap[y][x] != 3 && TileManager.tileMap[y][x] != 4) {
            return true;
        }
        return false;
    }

    @Override
    public void attack(IEntity entity, GraphicsContext graphicsContext) {
        BoundingBox hitBox;

        int xTile = mapPosX / gameSceneController.getScaledTileSize();
        int yTile = mapPosY / gameSceneController.getScaledTileSize();

        double dropChance = 0.5;
        double randomDropChance = Math.random() * 1;

        //Added and subtracted numbers from variables are the pixel insets of the player sprite

        int attackRange = 30;
        int attackWidth = 10;

        if (inputHandler.isAttack() && PLAYER_MODE == PlayerMode.PLAYER_ONE ||
                inputHandler.isSdAttack() && PLAYER_MODE == PlayerMode.PLAYER_TWO) {

            soundEffects.playSoundEffectOnce(SFX.SWORD_SWING);
            if (playerDirection == EntityDirection.UP) {
                hitBox = new BoundingBox(mapPosX + playerWidth, mapPosY - 10, attackWidth, attackRange);
                graphicsContext.strokeRect(mapPosX + playerWidth, mapPosY - 10, attackWidth, attackRange);
                graphicsContext.drawImage(swordUp, mapPosX + 8, mapPosY - 10, 32, 32);
                hitEnemy(entity, graphicsContext, hitBox, dropChance, randomDropChance);
            } else if (playerDirection == EntityDirection.DOWN) {
                hitBox = new BoundingBox(mapPosX + playerWidth, mapPosY + playerHeight, attackWidth, attackRange);
                graphicsContext.strokeRect(mapPosX + playerWidth, mapPosY + playerHeight, attackWidth, attackRange);
                graphicsContext.drawImage(swordDown, mapPosX + 8, mapPosY + playerHeight, 32, 32);
                hitEnemy(entity, graphicsContext, hitBox, dropChance, randomDropChance);
            } else if (playerDirection == EntityDirection.LEFT) {
                hitBox = new BoundingBox(mapPosX - attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.strokeRect(mapPosX - attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.drawImage(swordLeft, mapPosX - 8, mapPosY + 8, 32, 32);
                hitEnemy(entity, graphicsContext, hitBox, dropChance, randomDropChance);
            } else {
                hitBox = new BoundingBox(mapPosX + playerWidth + attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.strokeRect(mapPosX + playerWidth + attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.drawImage(swordRight, mapPosX + playerWidth + 8, mapPosY + 8, 32, 32);
                hitEnemy(entity, graphicsContext, hitBox, dropChance, randomDropChance);
            }
        }
    }

    private void hitEnemy(IEntity entity, GraphicsContext graphicsContext, BoundingBox hitBox, double dropChance, double randomDropChance) {
        if (hitBox.intersects(entity.getBoxCollider())) {
            soundEffects.playSoundEffectOnce(SFX.SWORD_HIT);
            entity.gotHit(damage);
            log.info("Hit enemy");

            //spawning heart entity
            if (randomDropChance > dropChance) {
                int spawnCordX = mapPosX + 48;
                int spawnCordY = mapPosY + 48;

                List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
                IEntity heart = EntityFactory.createGameplayObject(ObjectType.HEART, spawnCordX, spawnCordY, graphicsContext);
                gameplayObjects.add(heart);
                runtimeInfo.setGameplayObjects(gameplayObjects);
                log.info("heart dropped");
            }
        }
    }

    @Override
    public void updateEntityWalkAnimation() {
        spriteCounter++;

        if (spriteCounter > 10) {
            if (spriteNumber == 1) {
                spriteNumber = 2;
            } else if (spriteNumber == 2) {
                spriteNumber = 1;
            }
            spriteCounter = 0;
        }
    }

    @Override
    public void renderEntity(GraphicsContext graphicsContext) {
        Image playerSprite = null;

        //for debugging collision
        graphicsContext.strokeRect(mapPosX + 15, mapPosY + 10, playerWidth, playerHeight);

        switch (playerDirection) {
            case UP:
                if (spriteNumber == 1) {
                    playerSprite = directionUpOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionUpTwo;
                }
                break;
            case DOWN:
                if (spriteNumber == 1) {
                    playerSprite = directionDownOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionDownTwo;
                }
                break;
            case LEFT:
                if (spriteNumber == 1) {
                    playerSprite = directionLeftOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionLeftTwo;
                }
                break;
            case RIGHT:
                if (spriteNumber == 1) {
                    playerSprite = directionRightOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionRightTwo;
                }
                break;
        }

        graphicsContext.drawImage(playerSprite, mapPosX, mapPosY, 48, 48);
    }

    @Override
    public void gotHit(int damageDone) {
        health -= damageDone;
    }

    @Override
    public void healPlayer(int healthRegenerated) {
        int regeneratedHealth = health + healthRegenerated;
        soundEffects.playSoundEffectOnce(SFX.POWERUP);
        if (regeneratedHealth < maxPlayerHealth) {
            health += healthRegenerated;
        } else {
            health = maxPlayerHealth;
        }
        log.info(health);
    }

    @Override
    public BoundingBox getBoxCollider() {
        return boxCollider;
    }

    @Override
    public EntityDirection getEntityDirection() {
        return playerDirection;
    }

    @Override
    public int getEntitySpeed() {
        return playerSpeed;
    }

    @Override
    public int getMapPosX() {
        return mapPosX;
    }

    @Override
    public int getMapPosY() {
        return mapPosY;
    }

    @Override
    public ObjectType getOBJECT_TYPE() {
        return null;
    }

    @Override
    public ObjectStatus getOBJECT_STATUS() {
        return null;
    }

    @Override
    public void setObjectStatus(ObjectStatus objectStatus) {

    }

}