package de.hdm_stuttgart.battlearena.Model.Map;

import de.hdm_stuttgart.battlearena.Model.Sound.SFXLoop;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import java.util.Objects;

public class TileManager {

    private static final Logger log = LogManager.getLogger(TileManager.class);

    private final GraphicsContext graphicsContext2D;
    private final ITile[] tileSet;
    public static int[][] tileMap;

    private final int horizontalTileCount;
    private final int verticalTileCount;

    public TileManager(GraphicsContext graphicsContext2D, int diffTileCount,
                       int horizontalTileCount, int verticalTileCount, String mapString, Biom biom) {
        this.graphicsContext2D = graphicsContext2D;
        this.horizontalTileCount = horizontalTileCount;
        this.verticalTileCount = verticalTileCount;

        tileSet = new BackgroundTile[diffTileCount];
        tileMap = new int[horizontalTileCount][verticalTileCount];

        createTiles(biom);
        generateMap(mapString, horizontalTileCount, verticalTileCount);
    }

    //index 0-9: WALKABLE + NON_DESTRUCTIBLE, index 10-19: NON_WALKABLE + NON_DESTRUCTIBLE, index 20-29: DESTRUCTIBLE
    private void createTiles(Biom biom) {
        if (biom == Biom.GRASS) {
            try {
                //Textures of walkable tiles
                tileSet[0] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/ground/GrassGrassLand00.png"))), SFXLoop.GRASS);
                tileSet[1] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/ground/GrassGrassLand01.png"))), SFXLoop.GRASS);
                tileSet[2] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/ground/GrassPathGrassLand02.png"))), SFXLoop.GRASS);
                tileSet[3] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/ground/GrassGrassLand03.png"))), SFXLoop.GRASS);
                tileSet[4] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[5] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[6] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[7] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[8] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[9] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                //Textures of solid tiles + non destructible
                tileSet[10] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/building/StoneGrassLand00.png"))), SFXLoop.GRASS);
                tileSet[11] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/building/StoneEndLeftGrassLand01.png"))), SFXLoop.GRASS);
                tileSet[12] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/building/StoneEndRightGrassLand02.png"))), SFXLoop.GRASS);
                tileSet[13] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/building/StoneWindowGrassLand03.png"))), SFXLoop.GRASS);
                tileSet[14] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/building/StoneGateGrassLand04.png"))), SFXLoop.GRASS);
                tileSet[15] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/building/StoneGrassLand05.png"))), SFXLoop.GRASS);
                tileSet[16] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/grass_biom/ground/GrassGrassLand04.png"))), SFXLoop.GRASS);
                tileSet[17] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.GRASS);
                tileSet[18] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.GRASS);
                tileSet[19] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.GRASS);
                //Testures of destructible tiles
                tileSet[20] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[21] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[22] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[23] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[24] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[25] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[26] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[27] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[28] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[29] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
            } catch (Exception e) {
                log.error(e);
            }
        }else if(biom == Biom.DARK_LANDS){
            try {
                //Textures of walkable tiles
                tileSet[0] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/ground/GrassDarkLand00.png"))), SFXLoop.NONE);
                tileSet[1] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/ground/GrassDarkLand01.png"))), SFXLoop.NONE);
                tileSet[2] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/ground/GrassPathDarkLand02.png"))), SFXLoop.NONE);
                tileSet[3] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/ground/GrassDarkLand03.png"))), SFXLoop.NONE);
                tileSet[4] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[5] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[6] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[7] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[8] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[9] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                //Textures of solid tiles
                tileSet[10] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/building/StoneDarkLand00.png"))), SFXLoop.NONE);
                tileSet[11] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/building/StoneEndLeftDarkLand01.png"))), SFXLoop.NONE);
                tileSet[12] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/building/StoneEndRightDarkLand02.png"))), SFXLoop.NONE);
                tileSet[13] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/building/StoneWindowDarkLand03.png"))), SFXLoop.NONE);
                tileSet[14] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/building/StoneGateDarkLand04.png"))), SFXLoop.NONE);
                tileSet[15] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/building/NoTexture.png"))), SFXLoop.NONE);
                tileSet[16] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/dark_lands_biom/ground/NoTexture.png"))), SFXLoop.NONE);
                tileSet[17] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[18] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[19] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                //Testures of destructible tiles
                tileSet[20] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[21] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[22] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[23] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[24] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[25] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[26] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[27] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[28] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                tileSet[29] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                        new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
        } catch (Exception e) {
                log.error(e);
            }
        }else if(biom == Biom.BOMBERMAN){
                try {
                    //Textures of walkable tiles
                    tileSet[0] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/bomberman/ground/GrassBombLand00.png"))),SFXLoop.GRASS);
                    tileSet[1] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/bomberman/ground/GrassBombLand01.png"))), SFXLoop.NONE);
                    tileSet[2] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))),SFXLoop.NONE);
                    tileSet[3] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/bomberman/ground/GrassBombLand03.png"))), SFXLoop.NONE);
                    tileSet[4] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/bomberman/ground/GrassBombLand04.png"))),SFXLoop.NONE);
                    tileSet[5] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[6] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[7] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[8] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[9] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    //Textures of solid tiles
                    tileSet[10] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/bomberman/building/StoneBombLand00.png"))), SFXLoop.NONE);
                    tileSet[11] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/bomberman/building/StoneBombLand01.png"))), SFXLoop.NONE);
                    tileSet[12] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[13] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[14] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[15] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[16] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[17] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[18] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[19] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    //Testures of destructible tiles
                    tileSet[20] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/bomberman/building/box.png"))), SFXLoop.NONE);
                    tileSet[21] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[22] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[23] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[24] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[25] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[26] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[27] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[28] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[29] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                } catch (Exception e) {
                    log.error(e);
                }
        }else if(biom == Biom.FORREST){
                try {
                    //Textures of walkable tiles
                    tileSet[0] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/forrest_biom/ground/GrassWoodLand00.png"))), SFXLoop.NONE);
                    tileSet[1] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/forrest_biom/ground/GrassWoodLand01.png"))), SFXLoop.NONE);
                    tileSet[2] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/forrest_biom/ground/GrassPathWoodLand02.png"))), SFXLoop.NONE);
                    tileSet[3] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/forrest_biom/ground/GrassWoodLand03.png"))), SFXLoop.NONE);
                    tileSet[4] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[5] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[6] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[7] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[8] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[9] = TileFactory.createTile(TileType.WALKABLE, TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    //Textures of solid tiles
                    tileSet[10] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[11] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[12] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[13] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[14] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[15] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[16] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[17] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[18] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[19] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    //Testures of destructible tiles
                    tileSet[20] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[21] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[22] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[23] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[24] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[25] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[26] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[27] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[28] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                    tileSet[29] = TileFactory.createTile(TileType.NON_WALKABLE,TileType.NON_DESTRUCTIBLE,
                            new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/map/NoTexture.png"))), SFXLoop.NONE);
                } catch (Exception e) {
                    log.error(e);
                }
        }
    }

    private void generateMap(String mapString, int horizontalTileCount, int verticalTileCount) {
        String[] tileSet = mapString.split(" ");

        int stringIndex = 0;

        for (int column = 0; column < horizontalTileCount; column++) {
            for (int row = 0; row < verticalTileCount; row++) {
                tileMap[column][row] = Integer.parseInt(tileSet[stringIndex]);
                stringIndex++;
            }
        }
    }

    public void renderMap() {
        int currentTile;
        int scaledTileSize = 48; //todo: later replace "48" with value from gameScene.
        int tilePosX;
        int tilePosY;

        for (int mapColumn = 0; mapColumn < horizontalTileCount; mapColumn++) {
            for (int mapRow = 0; mapRow < verticalTileCount; mapRow++) {
                currentTile = tileMap[mapRow][mapColumn];

                tilePosX = mapColumn * scaledTileSize;
                tilePosY = mapRow * scaledTileSize;

                graphicsContext2D.drawImage(tileSet[currentTile].getTileSprite(), tilePosX, tilePosY,
                        scaledTileSize, scaledTileSize);
            }
        }
    }

    public int[][] getTileMap() {
        return tileMap;
    }

    public ITile[] getTileSet() {
        return tileSet;
    }
}